;	LCDisplayer 1.1
;	Hitachi LCD Controller..
;	
;
;  any hitachi LCD Instruction can be set after sending "254" (one byte) first
;   backlit  adjustment:  254 , 17 , value (0..255)
;   contrast adjustment:  254 , 18 , value (0..255) 


  LIST P=16F876, R=DEC          ; Runs at 4 MHz
  INCLUDE "p16f876.inc"


;  Register Usage
 CBLOCK 0x020                   ;  Start Registers at End of the Values
Dlay                            ;  8 Bit Delay Variable
Temp				;  Temporary Value Used When Sending Out Data
NOTemp				;  Temporary Value to "NybbleOutput"
Counter1
Counter2
Counter3
w_temp				; Working is stored here during Interrupt
status_temp			; Status  is stored here during Interrupt
Misc_Flags			;bit0  set if next incoming byte is an Instruction  
				;bit1  set if next incoming byte is Backlit value
				;bit2  set if next incoming byte is Contrast value  
Inst				;Value of instruction command =d'254' 
LastByte			;last incoming byte (FIFO can only be read one time)
 ENDC


;  Define Inforation
#DEFINE Data PORTA,0
#DEFINE Clock PORTA, 1
#DEFINE Backlit PORTC, 2
#DEFINE Contrast PORTC, 1

;  Macros
ClockStrobe MACRO		;  Strobe the Data Bit
  bsf	 Clock
  bcf	 Clock
 ENDM

EStrobe MACRO			;  Strobe the "E" Bit
  bsf	 Data
  bcf	 Data
 ENDM


 PAGE
 	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_ENABLE_ON & _LVP_OFF & _DEBUG_OFF & _CPD_OFF
                                

  org    0
	clrf    PCLATH            ; ensure page bits are cleared
	goto Start_Up


;--------------------------------------------------------------------------
;---------------INTERRUPT--------------------------------------------------
;--------------------------------------------------------------------------

	ORG     0x004             ; interrupt vector location
	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register


	movfw RCREG 		;get new byte from fifo
	movwf LastByte		; store in a perm. register


	btfsc Misc_Flags,0        ;Test if this byte should be treated as INSTRUCTION to LCD
	goto Incoming_INSTRUCTION
	
	movlw d'254' 		; this should be recognized as a instruction
	xorwf LastByte,w	; will result in 0 if next command shuold be a instruction
	btfsc STATUS,Z		; skip if normal character (not equal to 254)
	goto Incoming_254	; this is only information that next char is instruction
	movfw LastByte	
	call SendCHAR		; Output the ASCII Character
	goto Return_From_INT



Incoming_254
	bsf Misc_Flags,0	;next incoming byte will be a instruction to LCD.
	goto Return_From_INT


Incoming_INSTRUCTION		; instruction to LCD...

	btfsc Misc_Flags,1   ;Is it a Backlit Value ?
	goto Incoming_BACKLIT_VALUE

	btfsc Misc_Flags,2   ;Is it a Contrast Value ?
	goto Incoming_CONTRAST_VALUE


	movlw d'17' 		; check if it's a BACKLIT instruction
	xorwf LastByte,w		
	btfsc STATUS,Z		
	goto Incoming_BACKLIT_CMD	

	movlw d'18' 		; check if it's a CONTRAST instruction
	xorwf LastByte,w		
	btfsc STATUS,Z		
	goto Incoming_CONTRAST_CMD	


	movfw LastByte		;Get last incoming byte...
	call	 SendINS	;Output the as Instruction
	bcf Misc_Flags,0	;next incoming byte will be a CHARACTER
	goto Return_From_INT

Incoming_BACKLIT_CMD

	bsf Misc_Flags,1
	goto Return_From_INT

Incoming_CONTRAST_CMD

	bsf Misc_Flags,2
	goto Return_From_INT


Incoming_BACKLIT_VALUE
	bcf Misc_Flags,1   	;Next time it's no value
	bcf Misc_Flags,0	;next incoming byte will be a CHARACTER
	movfw LastByte
	MOVWF CCPR1L 		; Set Duty Cycle for Backlit
	goto Return_From_INT
	


Incoming_CONTRAST_VALUE
	bcf Misc_Flags,2   	;Next time it's no value
	bcf Misc_Flags,0	;next incoming byte will be a CHARACTER
	movfw LastByte
	MOVWF CCPR2L 		; Set Duty Cycle for Contrast
	goto Return_From_INT




Return_From_INT
	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt



;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------


Start_Up

  call	Wait_300_ms
  clrf   PORTA
  clrf   Misc_Flags


	bsf STATUS,RP0	; Bank 1 Selected 
	
;	bcf TRISA,RA0		;  LCD DATA
;	bcf TRISA,RA1		;  LCD Clock
;	bsf TRISA,RA2		; 
;	bsf TRISA,RA3		; 
;	bsf TRISA,RA4		; 

	movlw   b'11111100'
	movwf   TRISA                  ; 

;	bsf TRISB,RB0		; 
;	bsf TRISB,RB1		; 
;	bsf TRISB,RB2		; 
;	bsf TRISB,RB3		; 
;	bsf TRISB,RB4		; 
;	bsf TRISB,RB5		; 
;	bsf TRISB,RB6		; 
;	bsf TRISB,RB7		; 

	movlw   b'11111111'
	movwf   TRISB                  ; 

;	bsf TRISC,RC0		; 
;	bcf TRISC,RC1		; PWM2  (Contrast)
;	bcf TRISC,RC2		; PWM1  (Backlit)
;	bsf TRISC,RC3		;  
;	bsf TRISC,RC4		; 
;	bsf TRISC,RC5		; 
;	bcf TRISC,RC6		; TX  (RS232)
;	bsf TRISC,RC7		; RX  (RS232)

	movlw   b'10111001'
	movwf   TRISC                  ; 

	bcf OPTION_REG,NOT_RBPU		; DISABLE Weak Pullup when bit set
	bcf STATUS,RP0	; Bank 0 Selected 

	;;;bsf Backlit  ;turn on backlit  
	;;;;bcf Contrast ;maximum Contrast!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!




Initialize_PWM_Backlit
	CLRF CCP1CON ; CCP Module is off
	CLRF TMR2 ; Clear Timer2
	MOVLW 0xFf ;
	MOVWF PR2    ; Timer2's module period register  
	MOVLW 0xFF ;
	MOVWF CCPR1L ; Duty Cycle is 25% of PWM Period


	bcf INTCON,T0IF ; and clear int T0IF
	BSF STATUS, RP0 ; Bank1
	bcf PIE1,TMR2IE ; TMR2 equal to PR2 match int disable
	BCF STATUS, RP0 ; Bank0
	bcf PIR1,TMR2IF ; Clear interrupts Flag
	MOVLW 0x2C ; PWM mode, 2 LSbs of Duty cycle = 10
	MOVWF CCP1CON ;
	bsf T2CON,T2CKPS1 
	BSF T2CON, TMR2ON ; Timer2 starts to increment


Initialize_PWM_Contrast
	CLRF CCP2CON ; CCP Module is off
	MOVLW 0x85 ;
	MOVWF CCPR2L ; Duty Cycle is 25% of PWM Period
	MOVLW 0x2C ; PWM mode, 2 LSbs of Duty cycle = 10
	MOVWF CCP2CON ;


	
	call Initialize_LCD	;initialize Interface
	call Showoff 		;Show welcone strings
	


Initialize_RS232
	bsf STATUS,RP0	; Bank 1 Selected 
	bcf TXSTA,TX9   ; 8bit mode
	bcf TXSTA,SYNC  ; asyncron
	bsf TXSTA,BRGH  ; high speed
	movlw d'12'     ; 12=19200 , 25=9600 
	movwf SPBRG     ; 19200 baud
	bsf TXSTA,TXEN  ;Enable transmission
	bcf STATUS,RP0	; Bank 0 Selected 

	clrf RCSTA		
	bsf RCSTA,SPEN  ; Serial port enabled
	bsf RCSTA,CREN  ; continious recieve enabled





Enable_Interupts
	bsf STATUS,RP0	; Bank 1 Selected 
	clrf PIE2   ;no eeprom interrupts
	clrf PIE1   ;clear all periphial int's
	bsf PIE1,RCIE ;want a interrupt on recieve   
	bcf STATUS,RP0	; Bank 0 Selected 

	clrf PIR1   ;clear all flags
	clrf PIR2   ;clear all flags
	clrf INTCON
	bsf INTCON,PEIE 
	bsf INTCON,GIE  
	
Main_Loop

	movlw d'255' 
	xorwf PORTB,w		; check if all inputs are high (no buttons depressed)
	btfsc STATUS,Z		
	goto Main_Loop          ; No buttons depressed

	;Button pressed...

	movfw PORTB
	movwf TXREG   ;Transmit


Wait_For_Button_Release					
	;Prepare Time Delay Countdown for autorepeate= ,300002 s with Osc = 4 MHz
	movlw	D'2'
	movwf	Counter3
	movlw	D'130'
	movwf	Counter2
	movlw	D'10'
	movwf	Counter1
	
Loop_Autorep_Delay
	
	movlw d'255' 
	xorwf PORTB,w		; check if all inputs are high (no buttons depressed)
	btfsc STATUS,Z		
	goto Button_Released
				
	decfsz	Counter1,1		;loop
	goto	Loop_Autorep_Delay	;loop
	decfsz	Counter2,1		;loop
	goto	Loop_Autorep_Delay	;loop
	decfsz	Counter3,1		;loop
	goto	Loop_Autorep_Delay	;loop

;----------Enter autorepeat	

Autorepeat_Loop
	movlw d'255' 
	xorwf PORTB,w		; check if all inputs are high (no buttons depressed)
	btfsc STATUS,Z		
	goto Button_Released    ;if released , goto start prevent BOUNCING

	movfw PORTB
	movwf TXREG   ;Transmit

	Call Wait_200_ms

	goto Autorepeat_Loop


Button_Released
	call Dlay160	;debounce
	call Dlay160	;debounce

	
	goto Main_Loop         







;################################################################################
;################################################################################
;################################################################################
Initialize_LCD
  call   Dlay5                  ;  Wait 20 msecs before Reset
  call   Dlay5
  call   Dlay5
  call   Dlay5

  bcf    STATUS, C              ;  Clear Carry (Instruction Out)
  movlw  0x03                   ;  Reset Command
  call   NybbleOut              ;  Send the Nybble
  call	 Dlay5			;  Wait 5 msecs before Sending Again

  EStrobe
  call	 Dlay160		;  Wait 160 usecs before Sending the Third Time

  EStrobe
  call	 Dlay160		;  Wait 160 usecs before Sending the Third Time

  bcf    STATUS, C              
  movlw  0x02                   ;  Set 4 Bit Mode
  call   NybbleOut              
  call	 Dlay160		

  movlw	 0x028			;  2 Line Display
  call	 SendINS

  movlw	 0x008			;  Turn off the Display
  call	 SendINS

  movlw	 0x001			;  Clear the Display RAM
  call	 SendINS
  call	 Dlay5			;  Note, Can take up to 4.1 msecs

  movlw	 0x006			;  Enable Cursor Move Direction
  call	 SendINS

  movlw	 0x00F		;C is no cursor	;  Turn the LCD Back On
  call	 SendINS

	return


;################################################################################
;################################################################################
;################################################################################
Showoff
  clrf	 FSR			;  Output the Message
OutLoop
  movf	 FSR, w			;  Get the Offset to Output
  incf	 FSR
  call	 Message
  iorlw	 0			;  At the End of the Message?
  btfsc	 STATUS, Z
   goto	 Showoff2		;  Yes - Equal to Zero
  call	 SendCHAR		;  Output the ASCII Character
  goto	 OutLoop


	


Showoff2

	movlw  d'192'  ;(second Line)
	call	SendINS


  clrf	 FSR			;  Output the Message
OutLoop2
  movf	 FSR, w			;  Get the Offset to Output
  incf	 FSR
  call	 Message2
  iorlw	 0			;  At the End of the Message?
  btfsc	 STATUS, Z
   goto	 Showoff_Done		;  Yes - Equal to Zero
  call	 SendCHAR		;  Output the ASCII Character
  goto	 OutLoop2

Showoff_Done
	Call Wait_300_ms
	Call Wait_300_ms
	Call Wait_300_ms
	Call Wait_300_ms
	Call Wait_300_ms
	movlw  d'1'  ;(Blank Screen)
	call	 SendINS

	return	


Message				;  Message to Output
  addwf	 PCL			;  Output the Characters
  dt	 "LCD Control 1.1 ", 0

Message2			;  Message to Output
  addwf	 PCL			;  Output the Characters
  dt	 "(c) Andre Deyk ", 0





	


;################################################################################
;################################################################################
;################################################################################
SendCHAR			;  Send the Character to the LCD
  movwf	 Temp			;  Save the Temporary Value

  swapf	 Temp, w		;  Send the High Nybble
  bsf	 STATUS, C		;  RS = 1
  call	 NybbleOut

  movf	 Temp, w		;  Send the Low Nybble
  bsf	 STATUS, C
  call	 NybbleOut

  return

;################################################################################
;################################################################################
;################################################################################
SendINS				;  Send the Instruction to the LCD
  movwf	 Temp			;  Save the Temporary Value

  swapf	 Temp, w		;  Send the High Nybble
  bcf	 STATUS, C		;  RS = 0
  call	 NybbleOut

  movf	 Temp, w		;  Send the Low Nybble
  bcf	 STATUS, C
  call	 NybbleOut

  return

;################################################################################
;################################################################################
;################################################################################
NybbleOut			;  Send a Nybble to the LCD

  movwf	 NOTemp			;  Save the Nybble to Shift Out
  swapf	 NOTemp			;  Setup to Output to the High Part of the Byte

  movlw	 6			;  Clear the Shift Register
  movwf	 Dlay
NOLoop1
  ClockStrobe
  decfsz Dlay
   goto	 NOLoop1

  bsf	 Data			;  Put out the Gate Bit
  ClockStrobe
  bcf	 Data			;  Put out the RS Bit
  rlf	 PORTA
  ClockStrobe
  movlw	 4			;  Now, Shift out the Data
  movwf	 Dlay
NOLoop2
  rlf	 NOTemp			;  Shift Through the Nybble to Output
  bcf	 Data			;  Clear the Data Bit (which is the Clock)
  rlf	 PORTA			;  Shift the Carry into the Shift Register
  ClockStrobe
  decfsz Dlay
   goto	 NOLoop2

  EStrobe			;  Strobe out the LCD Data

  return

;################################################################################
;################################################################################
;################################################################################
Dlay160                         ;  Delay 160 usecs

  movlw  256 - ( 160 / 4 )      ;  Loop Until Carry Set
  addlw  1
  btfss  STATUS, C
   goto  $-2

  return

;################################################################################
;################################################################################
;################################################################################
Dlay5                           ;  Delay 5 msecs

  movlw  4                      ;  Set up the Delay
  movwf  Dlay
  movlw  256 - 0x0E8
  addlw  1
  btfsc  STATUS, Z
   decfsz Dlay
    goto $-3

  return






;################################################################################
;################################################################################
;################################################################################
Wait_200_ms
;Time Delay = ,200003 s with Osc = 4,000000 MHz
	movlw	D'2'
	movwf	Counter3
	movlw	D'5'
	movwf	Counter2
	movlw	D'185'
	movwf	Counter1
	goto Wait_loop		
	
;**************************************************************************
;**************************************************************************	
Wait_300_ms
;Time Delay = ,300003 s with Osc = 4 MHz
	movlw	D'2'
	movwf	Counter3
	movlw	D'135'
	movwf	Counter2
	movlw	D'153'
	movwf	Counter1
	goto Wait_loop		
	
;**************************************************************************
;**************************************************************************	
Wait_loop			;Universal Wait Loop
	decfsz	Counter1,F
	goto	Wait_loop
	decfsz	Counter2,F
	goto	Wait_loop
	decfsz	Counter3,F
	goto	Wait_loop
	return	




  end

